CREATE OR REPLACE PROCEDURE putline (approach IN VARCHAR2, ol PLS_INTEGER)
IS
   t0          INTEGER;
   t1          INTEGER;

   CURSOR cur
   IS
      SELECT *
        FROM all_source;

   one_row     cur%ROWTYPE;

   TYPE t IS TABLE OF cur%ROWTYPE
      INDEX BY PLS_INTEGER;

   many_rows   t;
BEGIN
   t0 := DBMS_UTILITY.get_cpu_time ();

   CASE approach
      WHEN 'implicit for loop'
      THEN
         FOR j IN cur
         LOOP
            NULL;
         END LOOP;
      WHEN 'explicit open, fetch, close'
      THEN
         OPEN cur;

         LOOP
            FETCH cur
             INTO one_row;

            EXIT WHEN cur%NOTFOUND;
         END LOOP;

         CLOSE cur;
      WHEN 'bulk fetch'
      THEN
         OPEN cur;

         LOOP
            FETCH cur
            BULK COLLECT INTO many_rows LIMIT 100;

            EXIT WHEN many_rows.COUNT () < 1;
         END LOOP;

         CLOSE cur;
   END CASE;

   t1 := DBMS_UTILITY.get_cpu_time ();
   DBMS_OUTPUT.put_line (   'Timing for '
                         || approach
                         || ' with opt level '
                         || TO_CHAR (ol)
                         || ' = '
                         || TO_CHAR (t1 - t0)
                        );
END putline;
/
SET FEEDBACK OFF
--------------------------------------------------------------------------------
-- Time it at optimize levels 1 and 2 
--
-- Level 1


alter procedure putline compile plsql_optimize_level=1 
/
call putline( 'implicit for loop' , 1)          -- 2073
/
call putline( 'explicit open, fetch, close' , 1) -- 2063
/
call putline( 'bulk fetch' , 1)                  -- 252
/


--------------------------------------------------------------------------------
-- Level 2


alter procedure putline compile plsql_optimize_level=2 
/
call putline( 'implicit for loop' , 2)          -- 263 <<== NOTE THE CHANGE !
/
call putline( 'explicit open, fetch, close' , 2) -- as for level 1
/
call putline( 'bulk fetch' , 2)                  -- as for level 1
/

/*
Results on 10g Release 2

Timing for implicit for loop with opt level 1 = 793

Timing for explicit open, fetch, close with opt level 1 = 757

Timing for bulk fetch with opt level 1 = 376

Timing for implicit for loop with opt level 2 = 363

Timing for explicit open, fetch, close with opt level 2 = 747

Timing for bulk fetch with opt level 2 = 333
*/