CREATE OR REPLACE PROCEDURE nested_loop_static
IS
   v employee.last_name%TYPE;
BEGIN
   FOR drec IN (SELECT department_id, name FROM department)
   LOOP
      FOR erec IN (SELECT last_name FROM employee
                    WHERE department_id = drec.department_id)
      LOOP
	     v := erec.last_name;
      END LOOP;
   END LOOP;
END;
/
CREATE OR REPLACE PROCEDURE nested_loop_dynamic
IS
   v employee.last_name%TYPE;
   fdbk PLS_INTEGER;
   cur PLS_INTEGER := DBMS_SQL.OPEN_CURSOR;
BEGIN
   DBMS_SQL.PARSE (cur,
	 'SELECT last_name FROM employee ' ||
     ' WHERE department_id = :dept', DBMS_SQL.nATIVE);
   DBMS_SQL.DEFINE_COLUMN (cur, 1, 'a', 100);
   FOR drec IN (SELECT department_id, name FROM department)
   LOOP
      DBMS_SQL.BIND_VARIABLE (cur, 'dept', drec.department_id);
	  fdbk := DBMS_SQL.EXECUTE (cur);
      WHILE DBMS_SQL.FETCH_ROWS (cur) > 0
      LOOP
	     DBMS_SQL.COLUMN_VALUE (cur, 1, v);
      END LOOP;
   END LOOP;
   DBMS_SQL.CLOSE_CURSOR (cur);
END;
/
CREATE OR REPLACE PROCEDURE test_nested (num IN INTEGER)
IS
BEGIN
   PLVtmr.capture;
   FOR i IN 1 .. num
   LOOP
      nested_loop_static;
   END LOOP;
   PLVtmr.show_elapsed ('static');
   
   PLVtmr.capture;
   FOR i IN 1 .. num
   LOOP
      nested_loop_dynamic;
   END LOOP;
   PLVtmr.show_elapsed ('dynamic');
END;
/

