DROP TABLE parts;

CREATE TABLE parts (
   partnum NUMBER,
   partname VARCHAR2(15)
   );

CREATE OR REPLACE PROCEDURE compare_fetching (num IN INTEGER)
IS
   TYPE NumTab IS TABLE OF parts.partnum%TYPE INDEX BY BINARY_INTEGER;
   TYPE NameTab IS TABLE OF parts.partname%TYPE INDEX BY BINARY_INTEGER;
   pnums NumTab;
   pnames NameTab;
BEGIN
   EXECUTE IMMEDIATE 'TRUNCATE TABLE parts';
   
   /* Load up the table. */
   FOR indx IN 1..num
   LOOP
      INSERT INTO parts VALUES (indx, 'Part ' || TO_CHAR(indx));
   END LOOP;
   
   COMMIT;
   
   DBMS_SESSION.FREE_UNUSED_USER_MEMORY;
   
   /* Fetch the data row by row */
   PLVtmr.capture;
   FOR rec IN (SELECT * FROM parts)
   LOOP
      pnums(SQL%ROWCOUNT)  := rec.partnum;
      pnames(SQL%ROWCOUNT) := rec.partname;
   END LOOP;
   PLVtmr.show_elapsed ('Single row fetch '|| num);

   /* Clean up the in-memory data structures. */   
   pnums.DELETE;
   pnames.DELETE;
   DBMS_SESSION.FREE_UNUSED_USER_MEMORY;
   
   /* Fetch the data in bulk */
   PLVtmr.capture;
   SELECT * BULK COLLECT INTO pnums, pnames FROM parts;
   PLVtmr.show_elapsed ('BULK COLLECT '|| num);
END;
/
BEGIN
   compare_fetching (1000);
   compare_fetching (10000);
   compare_fetching (100000);
END;
/   
