CREATE OR REPLACE PROCEDURE log_error (
   msg_in IN VARCHAR2 DEFAULT NULL
  ,code_in IN PLS_INTEGER DEFAULT NULL
)
IS
BEGIN
   -- A "stub" program that simply displays the error information.
   DBMS_OUTPUT.put_line (
      'Error message: ' || 
      NVL (msg_in, DBMS_UTILITY.FORMAT_ERROR_STACK));
   DBMS_OUTPUT.put_line ('Error code: ' || NVL (code_in, SQLCODE));
END log_error;
/

CREATE OR REPLACE PROCEDURE upd_for_dept (
   dept_in   IN   employee.department_id%TYPE
 , newsal_in IN   employee.salary%TYPE
)
IS
   bulk_errors   EXCEPTION;
   PRAGMA EXCEPTION_INIT (bulk_errors, -24381);

   TYPE employee_tt IS TABLE OF employee.employee_id%TYPE
      INDEX BY BINARY_INTEGER;

   employee_ids     employee_tt;

   TYPE salary_tt IS TABLE OF employee.salary%TYPE
      INDEX BY BINARY_INTEGER;

   salaries      salary_tt;

   TYPE hire_date_tt IS TABLE OF employee.hire_date%TYPE
      INDEX BY BINARY_INTEGER;

   hire_dates    hire_date_tt;

   PROCEDURE insert_history
   IS
   BEGIN
      FORALL indx IN employee_ids.FIRST .. employee_ids.LAST 
      SAVE EXCEPTIONS
         INSERT INTO employee_history
                     (employee_id, salary, hire_date
                     )
              VALUES (employee_ids (indx), salaries (indx), hire_dates (indx)
                     );
   EXCEPTION
      WHEN bulk_errors
      THEN
         FOR indx IN 1 .. SQL%BULK_EXCEPTIONS.COUNT
         LOOP
		    -- Log the error
		    log_error('Unable to insert history row for employee '||
			             employee_ids (SQL%BULK_EXCEPTIONS (indx).ERROR_INDEX),
			           SQL%BULK_EXCEPTIONS (indx).ERROR_code); 
            -- Delete this row so that the update will not take place. 
            employee_ids.DELETE (SQL%BULK_EXCEPTIONS (indx).ERROR_INDEX);
         END LOOP;
   END insert_history;

   PROCEDURE update_employee
   IS
   BEGIN
      -- Use Oracle10g INDICES OF to use what may now be
      -- a sparsely-populated employee_ids collection.
      FORALL indx IN INDICES OF employee_ids SAVE EXCEPTIONS
         UPDATE employee
            SET salary = newsal_in
              , hire_date = hire_dates (indx)
          WHERE employee_id = employee_ids (indx);
   EXCEPTION
      WHEN bulk_errors
      THEN
         FOR indx IN 1 .. SQL%BULK_EXCEPTIONS.COUNT
         LOOP
            -- Now I simply log the error...
            log_error ('Unable to update salary for employee '||
			             employee_ids (SQL%BULK_EXCEPTIONS (indx).ERROR_INDEX),
			           SQL%BULK_EXCEPTIONS (indx).ERROR_code);
         END LOOP;
   END update_employee;
BEGIN
   SELECT     employee_id, salary, hire_date
   BULK COLLECT INTO employee_ids, salaries, hire_dates
         FROM employee
        WHERE department_id = dept_in
   FOR UPDATE;

   insert_history;
   update_employee;
END upd_for_dept;
/
