CREATE OR REPLACE PROCEDURE cfl_vs_bulkcollect (
   iterations_in   IN   PLS_INTEGER := 1
)
IS
   cfl_tmr       tmr_t       := tmr_t.make ('Cursor For Loop', iterations_in);
   bulk_tmr      tmr_t       := tmr_t.make ('Bulk Collect', iterations_in);

   TYPE employee_tt IS TABLE OF employee_big%ROWTYPE
      INDEX BY BINARY_INTEGER;

   l_employees   employee_tt;
BEGIN
   DBMS_SESSION.free_unused_user_memory;
   cfl_tmr.go;

   FOR indx IN 1 .. iterations_in
   LOOP
      FOR rec IN (SELECT *
                    FROM employee_big)
      LOOP
         l_employees (NVL (l_employees.LAST, 0) + 1) := rec;
      END LOOP;
   END LOOP;

   cfl_tmr.STOP;
   l_employees.DELETE;
   DBMS_SESSION.free_unused_user_memory;
   bulk_tmr.go;

   FOR indx IN 1 .. iterations_in
   LOOP
      SELECT *
      BULK COLLECT INTO l_employees
        FROM employee_big;
   END LOOP;

   bulk_tmr.STOP;
END;
/

BEGIN
   cfl_vs_bulkcollect;
END;
/
