CREATE OR REPLACE PROCEDURE exec_dynplsql (
   blk_in     IN   VARCHAR2
  ,trace_in   IN   BOOLEAN := FALSE
  ,reraise_in IN   BOOLEAN := TRUE
)
AUTHID CURRENT_USER
IS
   -- Guarantee a valid PL/SQL block.
   mycode   VARCHAR2 (32767) := 
      'BEGIN ' || RTRIM (blk_in, ';') || '; END;';
	  
   -- Local substitute for DBMS_OUTPUT.PUT_LINE
   PROCEDURE pl (str IN VARCHAR2, len IN INTEGER := 80)
   IS
      v_len PLS_INTEGER := LEAST (len, 255);
      v_len2 PLS_INTEGER;
      v_chr10 PLS_INTEGER;
   BEGIN
      IF LENGTH (str) > v_len
      THEN
         v_chr10 := INSTR (str, CHR (10));

         IF v_chr10 > 0 AND v_len < v_chr10
         THEN
            v_len :=   v_chr10 - 1;
            v_len2 :=   v_chr10 + 1;
         ELSE
            v_len :=   v_len - 1;
            v_len2 := v_len;
         END IF;

         DBMS_OUTPUT.put_line (SUBSTR (str ,1 ,v_len));
         pl (SUBSTR (str, v_len2), len);
      ELSE
         DBMS_OUTPUT.put_line (str);
      END IF;
   END;	  
BEGIN
   IF trace_in
   THEN
      pl (mycode);
   END IF;

   -- Assumes no bind variables...
   EXECUTE IMMEDIATE mycode;
EXCEPTION
   WHEN OTHERS
   THEN
      pl ('Dynamic PL/SQL Error!');
      pl ('Error is:');
      pl (DBMS_UTILITY.FORMAT_ERROR_STACK);
      pl ('On block:');
      pl (mycode);
	  IF reraise_in THEN RAISE; END IF;
END;
/
