# PACKAGE TO CREATE AND SEND EMAIL USING SEND MAIL.
# CREATES FLAT FILE READ BY A CRON JOB.
# LAST MODIFIED 09/28/1999 BOB RUDOLF
#

CREATE OR REPLACE PACKAGE SNDMAIL IS
/**********************************************************/
/* Different communication procedures                     */
/**********************************************************/
         /* Validation procedure  ASSERT                  */
        PROCEDURE ASSERT
        (bool_in IN BOOLEAN,
        stg_in  IN VARCHAR2 DEFAULT NULL);

        /* Procedure to send the e-mail message */
        -- Parameters:
        -- p_to - e-mail address of the receiver. It can be a list of
        --        adresses separated with ONE blank. Must be supplied!
        -- p_subject - subject of the message, can be NULL
        -- p_text  - text of the message - maximum 32767 Bytes, required! 
        -- Example: WISDM.SNDMAIL.SEND_MAIL(
        --              'myaddress@mail.sbic.co.za suljo@mail.sbic.co.za',
        --      'This is the subject',
        --              'This is the text of the message');
        PROCEDURE SEND_MAIL  
        (p_to IN VARCHAR2, p_subject IN VARCHAR2, p_text IN VARCHAR2);

        /* Procedure to send the SMS */
        -- Parameters:
        -- p_tel_no     - phone number or list of numbers
        --                separated with ONE blank
        -- p_text       - text of the message - maximum 140 Bytes
	-- 
        -- Example: WISDM.SNDMAIL.SEND_SMS(
        --              '0832789012 0824441234',
        --              'This is the text of the message'); 
        PROCEDURE SEND_SMS  
        (p_tel_no IN VARCHAR2,
         p_text IN VARCHAR2);

END;
/
CREATE OR REPLACE PACKAGE BODY SNDMAIL IS
/**********************************************************/
/* Different communication procedures                             */
/**********************************************************/
        /* Constants */
        mtn_code              	CONSTANT       VARCHAR2(3) := '083'; 
        vodacom_code     	CONSTANT       VARCHAR2(3) := '082';
        mtn_address            	CONSTANT       VARCHAR2(30) := '@sms.co.za';
        vodacom_address  	CONSTANT       VARCHAR2(30) := 'sms@yebo.co.za';
         
        PROCEDURE ASSERT
        (bool_in IN BOOLEAN,
         stg_in  IN VARCHAR2 DEFAULT NULL)
        IS
        BEGIN
                IF NOT bool_in OR bool_in IS NULL
                THEN
                        IF stg_in IS NOT NULL
                        THEN
                                DBMS_OUTPUT.PUT_LINE(stg_in);
                        END IF;
                        RAISE VALUE_ERROR;
                END IF; 
        END;

        /* Procedure to send the e-mail message */
        -- Parameters:
        -- p_to 	- e-mail address of the receiver. It can be a list of
        --        	  adresses separated with blank
        -- p_subject 	- subject of the message
        -- p_text  	- text of the message - maximum 32767 Bytes 
        -- Example: WISDM.SNDMAIL.SEND_MAIL(
        --              'myaddress@mail.sbic.co.za suljo@mail.sbic.co.za',
        --      	'This is the subject',
        --              'This is the text of the message'); 
        PROCEDURE SEND_MAIL  
        (p_to IN VARCHAR2,
         p_subject IN VARCHAR2,
         p_text IN VARCHAR2)
        IS
                v_subject       VARCHAR2(100);
                v_text          VARCHAR2(32767);
                file_handle     UTL_FILE.FILE_TYPE;
                v_dir           VARCHAR2(100);
                v_file          VARCHAR2(100);
                v_dat_file      VARCHAR2(100);
                v_start_text VARCHAR2(1000);
                v_txt_len       NUMBER;
                v_loop_cnt      NUMBER;
                seq_no          VARCHAR2(6);
                dat_file_ind    BOOLEAN := FALSE;
                
        BEGIN
                
                -- Check input
                ASSERT (p_to IS NOT NULL,
                                'Please supply recipient');
                ASSERT (p_text IS NOT NULL,
                                'Please supply text');
                ASSERT (NVL(LENGTH(p_subject),0) < 101,
                                'Subject to long - maximum 100 Bytes');
                ASSERT ((LENGTH(p_to) + LENGTH(p_subject) ) < 1000,
                                'Recipient to long - send 2 or more messages');
                IF (LENGTH(p_to) + LENGTH(v_subject) + LENGTH(p_text)) > 999
                THEN
                        dat_file_ind := TRUE;
                END IF;

                /* Set up directory and file name, usig sequence */
                SELECT TO_CHAR(WISDM.SEND_MAIL_FILE.NEXTVAL),
                           DECODE(p_subject,NULL,NULL,'-s '||WISDM.UTILITY.quoteme(p_subject))
                INTO seq_no,
                         v_subject
                FROM DUAL;

                v_dir := '/udd001/home/dmaint/wisdm_alert';
                v_file := 'mail'||seq_no||'.sh';
                v_dat_file := 'mail'||seq_no||'.dat';

                file_handle := UTL_FILE.FOPEN(v_dir,v_file,'w');
                IF dat_file_ind
                THEN
                        v_start_text := 'more '||v_dat_file;
                ELSE
                        v_start_text := 'echo '||WISDM.UTILITY.quoteme(p_text);
                END IF;

                UTL_FILE.PUT_LINE(file_handle,
                        v_start_text||
                        ' | mailx '|| v_subject ||' '||
                        WISDM.UTILITY.quoteme(p_to));
                IF UTL_FILE.IS_OPEN(file_handle)
                THEN
                        UTL_FILE.FCLOSE(file_handle);
                END IF;

                -- Send .dat file if message was to long
                IF dat_file_ind
                THEN
        --              v_text := p_text;
        --              v_txt_len := LENGTH(v_text);

                        v_txt_len := LENGTH(p_text);

                        v_loop_cnt := 1;
                        file_handle := UTL_FILE.FOPEN(v_dir,v_dat_file,'w');
                        LOOP
                                EXIT WHEN v_loop_cnt > v_txt_len;
                                UTL_FILE.PUT_LINE(file_handle,
                                                SUBSTR(p_text,v_loop_cnt,80));
                                v_loop_cnt := v_loop_cnt + 80;
                        END LOOP;

                        IF UTL_FILE.IS_OPEN(file_handle)
                        THEN
                                UTL_FILE.FCLOSE(file_handle);
                        END IF;
                END IF;

        EXCEPTION
                WHEN UTL_FILE.INVALID_PATH
                THEN DBMS_OUTPUT.PUT_LINE('INV PATH');
                         RAISE;
                WHEN UTL_FILE.invalid_mode
                THEN DBMS_OUTPUT.PUT_LINE('invalid_mode');
                         RAISE;
                WHEN UTL_FILE.invalid_filehandle
                THEN DBMS_OUTPUT.PUT_LINE('invalid_filehandle');
                         RAISE;
                WHEN UTL_FILE.invalid_operation
                THEN DBMS_OUTPUT.PUT_LINE('invalid_operation');
                         RAISE;
                WHEN UTL_FILE.write_error
                THEN DBMS_OUTPUT.PUT_LINE('write_error');
                         RAISE;

                WHEN UTL_FILE.read_error
                THEN DBMS_OUTPUT.PUT_LINE('read_error');
                         RAISE;

                WHEN UTL_FILE.internal_error
                THEN DBMS_OUTPUT.PUT_LINE('internal_error');
                         RAISE;

                WHEN OTHERS
                THEN DBMS_OUTPUT.PUT_LINE(SQLERRM);
                        RAISE;
        END;


        /* Procedure to send the SMS  */
        -- At the moment available only for MTN subscribers
        -- Parameters:
        -- p_tel_no       - cell phone number or list of numbers
        --              separated with blank
        -- p_text  - text of the message - maximum 140 Bytes 
        -- Example: WISDM.SNDMAIL.SEND_SMS(
        --              '0832789012 0834441234',
        --      'This is the subject',
        --              'This is the text of the message'); 
        PROCEDURE SEND_SMS  
        (p_tel_no IN VARCHAR2,
         p_text IN VARCHAR2)
        IS
                TYPE tel_type IS TABLE OF VARCHAR2(15) 
                        INDEX BY BINARY_INTEGER;
                tel_tab tel_type;
                j INTEGER;
                k INTEGER;
                pos INTEGER;
                v_code VARCHAR2(3);
                v_tel_no VARCHAR2(2000);
        BEGIN
                ASSERT (LENGTH(p_text) < 140,
                                'Message to long');
                v_tel_no := RTRIM(LTRIM(p_tel_no));
                ASSERT (v_tel_no IS NOT NULL,
                                'At least one telefon number must be supplied');
                
                pos := 1;
                k := 0;
                LOOP
                        j := INSTR(v_tel_no,' ',pos);
                        EXIT WHEN j=0;
                        k := k +1;
                        tel_tab(k) := SUBSTR(v_tel_no,pos,j-pos);
                        pos := j + 1;
                END LOOP;
                k := k +1;
                tel_tab(k) := SUBSTR(v_tel_no,pos);

                FOR i IN 1 .. k
                LOOP
                        v_code := SUBSTR(tel_tab(i),1,3);
                        IF v_code = mtn_code
                        THEN 
                                SEND_MAIL('++27'||SUBSTR(tel_tab(i),2)||mtn_address,
                                                NULL,p_text);
                        ELSIF v_code = vodacom_code
                        THEN 
                                SEND_MAIL(vodacom_address,tel_tab(i),p_text);
                        ELSE
                                DBMS_OUTPUT.PUT_LINE('Wrong provider code');
                                RAISE VALUE_ERROR;
                        END IF;
                END LOOP;
                        
        END;


END;
/
---------------------------------------------
-----------------------------------------------
-- Unix job which runs with crontab
---------------------------------------------------

#####################################################################
#                            STEP 1                                 #
# Get a list of all the mail SH files that are going to be executed #
# Get a list of all DAT files so that we can remove these later     #
#####################################################################
cd /udd001/home/dmaint/wisdm_alert
rm job_log
ls mail*.sh  >  all_sh_files
ls mail*.dat > all_dat_files
#####################################################################
#                            STEP 2                                 #
# Loop through the SH file list and Execute them                    #
#####################################################################
echo "THE FOLLOWING JOBS HAVE BEEN EXECUTED" > job_log
for sendmailx in `cat all_sh_files`
do
  chmod 744 $sendmailx
  $sendmailx
  wait
  echo "SH FILE " $sendmailx " HAS BEEN EXECUTED" >> job_log
done
#####################################################################
#                            STEP 3                                 #
# Generate and run REMOVE script to RM all the SH and DAT files     #
#####################################################################
cat all_sh_files  | awk '{print "rm "$1}' >  rm_all_old_sh_+_dat
cat all_dat_files | awk '{print "rm "$1}' >> rm_all_old_sh_+_dat
echo "THE FOLLOWING FILE HAVE BEEN REMOVED" >> job_log
cat rm_all_old_sh_+_dat >> job_log
chmod 744 rm_all_old_sh_+_dat
rm_all_old_sh_+_dat
rm rm_all_old_sh_+_dat
rm all_sh_files all_dat_files


