/* Show how serially reusable packages don't preserve global state
   and reinitialize each time.
*/

CREATE OR REPLACE PACKAGE demo
AS
   PRAGMA SERIALLY_REUSABLE;
   global_x PLS_INTEGER := 10;

   CURSOR emps_cur
   IS
      SELECT *
        FROM employees;
END;
/

BEGIN
   demo.global_x := 50;
   p.l ( demo.global_x );
END;
/

BEGIN
   p.l ( demo.global_x );
END;
/

CREATE OR REPLACE PACKAGE valerr
IS
   PRAGMA SERIALLY_REUSABLE;

   FUNCTION get
      RETURN VARCHAR2;
END valerr;
/

CREATE OR REPLACE PACKAGE BODY valerr
IS
   PRAGMA SERIALLY_REUSABLE;
   v VARCHAR2 ( 3 ) := 'ABC';

   FUNCTION get
      RETURN VARCHAR2
   IS
   BEGIN
      p.l ( 'getting v' );
      RETURN v;
   END;
BEGIN
   p.l ( 'Before I show you v...' );
EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.put_line ( 'Trapped the error!' );
END valerr;
/

BEGIN
   p.l ( valerr.get );
END;
/

BEGIN
   p.l ( valerr.get );
END;
/
