DROP  TYPE tickertype FORCE;
DROP  TYPE tickertypeset FORCE;
DROP  TABLE stocktable;
DROP  TABLE tickertable;

CREATE TABLE  stocktable (
  ticker VARCHAR2(20),
  trade_date DATE,
  open_price NUMBER,
  close_price NUMBER
);

BEGIN
   -- Populate the table.
   INSERT INTO stocktable
        VALUES ('ORCL', SYSDATE, 15.2, 15.7);

   INSERT INTO stocktable
        VALUES ('QSFT', SYSDATE, 13.1, 13.5);

   INSERT INTO stocktable
        VALUES ('MSFT', SYSDATE, 27, 27.04);

   FOR indx IN 1 .. 100000
   LOOP
      -- Might as well be optimistic!
      INSERT INTO stocktable
           VALUES ('STK' || indx, SYSDATE, indx, indx + 15);
   END LOOP;

   COMMIT;
END;
/

CREATE TABLE tickertable
(
  ticker VARCHAR2(20),
  pricedate DATE,
  pricetype VARCHAR2(1),
  price NUMBER
)
/

/* 
   Note: Must use a nested table or varray of objects
   for the return type of a pipelined function
*/

CREATE TYPE tickertype AS OBJECT (
   ticker      VARCHAR2 (20)
 , pricedate   DATE
 , pricetype   VARCHAR2 (1)
 , price       NUMBER
);
/

CREATE TYPE tickertypeset AS TABLE OF tickertype;
/

CREATE OR REPLACE PACKAGE refcur_pkg
IS
   TYPE refcur_t IS REF CURSOR
      RETURN stocktable%ROWTYPE;
END refcur_pkg;
/

CREATE OR REPLACE FUNCTION stockpivot (dataset refcur_pkg.refcur_t)
   RETURN tickertypeset
IS
   l_row_as_object    tickertype        := tickertype (NULL, NULL, NULL, NULL);
   l_row_from_query   dataset%ROWTYPE;
   retval             tickertypeset     := tickertypeset ();
BEGIN
   -- APPLICATION SPECIFIC LOGIC!
    
   LOOP
      FETCH dataset
       INTO l_row_from_query;

      EXIT WHEN dataset%NOTFOUND;
      --
      l_row_as_object.ticker := l_row_from_query.ticker;
      l_row_as_object.pricetype := 'O';
      l_row_as_object.price := l_row_from_query.open_price;
      l_row_as_object.pricedate := l_row_from_query.trade_date;
      retval.EXTEND;
      retval (retval.LAST) := l_row_as_object;
      --
      l_row_as_object.pricetype := 'C';
      l_row_as_object.price := l_row_from_query.close_price;
      l_row_as_object.pricedate := l_row_from_query.trade_date;
      retval.EXTEND;
      retval (retval.LAST) := l_row_as_object;
   END LOOP;

   CLOSE dataset;

   RETURN retval;
END stockpivot;
/

CREATE OR REPLACE FUNCTION stockpivot (dataset refcur_pkg.refcur_t)
   RETURN tickertypeset
IS
   out_obj    tickertype        := tickertype (NULL, NULL, NULL, NULL);
   --
   TYPE dataset_tt IS TABLE OF stocktable%ROWTYPE
      INDEX BY PLS_INTEGER;

   l_dataset   dataset_tt;
   retval     tickertypeset     := tickertypeset ();
   l_row pls_integer;
BEGIN
   FETCH dataset
   BULK COLLECT INTO l_dataset;
   CLOSE dataset;
   
   l_row := l_dataset.FIRST;

   WHILE (l_row IS NOT NULL)
   LOOP
      out_obj.ticker := l_dataset(l_row).ticker;
      out_obj.pricetype := 'O';
      out_obj.price := l_dataset(l_row).open_price;
      out_obj.pricedate := l_dataset(l_row).trade_date;
      retval.EXTEND;
      retval (retval.LAST) := out_obj;
	  --
      out_obj.pricetype := 'C';
      out_obj.price := l_dataset(l_row).close_price;
      out_obj.pricedate := l_dataset(l_row).trade_date;
      retval.EXTEND;
      retval (retval.LAST) := out_obj;
	  --
      l_row := l_dataset.NEXT (l_row);
   END LOOP;

   RETURN retval;
END;
/

BEGIN
   INSERT INTO tickertable
      SELECT *
        FROM TABLE (stockpivot (CURSOR (SELECT *
                                          FROM stocktable)));
END;
/

/* Example of multiple transformations */

BEGIN
  open cv for
      SELECT *
        FROM TABLE
                ( transformation_2
                       ( CURSOR
                               ( SELECT *
                                   FROM TABLE
                                           ( stockpivot
                                                       ( CURSOR
                                                               ( SELECT *
                                                                   FROM stocktable )
                                                       )
                                           )
                               )
                       )
                );
END;
/
